//
//  TiggerClockController.m
//  Tigger
//
//  Created by Andreas on Mon Sep 15 2003.
//  Copyright (c) 2003 Andreas Mayer. All rights reserved.
//
//	2004-07-11	Andreas Mayer
//	- remove last sign before adding new one
//	- moved to Tigger's new launch notification
//	2004-07-17	Andreas Mayer
//	- updated for Tigger 1.2


#import "TiggerClockController.h"
#import "AMTickerService/AMTextTickerSign.h"
#import "AMTickerService/AMImageTickerSign.h"


@interface TiggerClockController (Private)
- (void)findServer;
- (void)startTimer;
- (void)stopTimer;
- (void)tick:(id)sender;
@end

@implementation TiggerClockController

- (void)awakeFromNib
{
	_am_clockSymbol = [[NSString stringWithUTF8String:"◷"] retain];
	// findServer
	[self findServer];
	// we want to know when a new app is launched - it might be our server
	[[NSDistributedNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationLaunched:) name:AMTiggerDidLaunchNotification object:nil];
}

- (void)dealloc
{
	[_am_clockSymbol release];
	[super dealloc];
}

- (id)server
{
    return server;
}

- (void)setServer:(id)newServer
{
    id old = nil;

    if (newServer != (id)server) {
        old = server;
        server = [newServer retain];
        [old release];
    }
}

- (NSTimer *)timer
{
    return timer;
}

- (void)setTimer:(NSTimer *)newTimer
{
	id old = nil;
	
	if (newTimer != timer) {
		old = timer;
		timer = [newTimer retain];
		[old release];
	}
}


- (void)applicationLaunched:(NSNotification *)aNotification
{
	NSLog(@"application launched: %@", [[aNotification userInfo] objectForKey:@"NSApplicationBundleIdentifier"]);
	if ([[[aNotification userInfo] objectForKey:@"NSApplicationBundleIdentifier"] isEqualToString:@"de.furrysoft.Tigger"]) {
		[self performSelector:@selector(findServer) withObject:nil afterDelay:1.0];
	}
}

- (void)setupWithServer:(id)theServer
{
	if (theServer != (id)server) {
		NSLog(@"wrong server responded?!");
	}
}

- (void)activate
{
	[self startTimer];
}

- (void)deactivate
{
	//NSLog(@"deactivate");
	[self stopTimer];
	[self setServer:nil];
}

- (BOOL)hasPrefsPane
{
	return NO;
}


- (void)findServer
{
	//NSLog(@"findServer");
	if (server == nil) { // only if we do not already have a valid server
		id theServer;
		theServer = [NSConnection rootProxyForConnectionWithRegisteredName:@"AMTiggerServer" host:nil];
		if (theServer) {
			[theServer setProtocolForProxy:@protocol(AMTiggerServerProtocol)];
			[theServer registerClient:self withName:@"AMTiggerClock"];
			[self setServer:theServer];
		} else {
			NSLog(@"error: could not connect to server");
		}
	} else {
		NSLog(@"already connected");
	}
}

- (void)startTimer
{
	float interval = 60.0;
	NSCalendarDate *startTime = [NSCalendarDate calendarDate];
	div_t divresult = div(([startTime secondOfMinute]+(int)interval), (int)interval);
	int startSecond = divresult.quot*(int)interval;
	//NSLog(@"start second for timer with interval %f: %i", interval, startSecond);
	startTime = [NSCalendarDate dateWithYear:[startTime yearOfCommonEra] month:[startTime monthOfYear] day:[startTime dayOfMonth] hour:[startTime hourOfDay] minute:[startTime minuteOfHour] second:startSecond timeZone:[startTime timeZone]];
	NSTimer *newTimer = [NSTimer alloc];
	if ([newTimer respondsToSelector:@selector(initWithFireDate:interval:target:selector:userInfo:repeats:)]) {
		newTimer = [[newTimer initWithFireDate:startTime interval:interval target:self selector:@selector(tick:) userInfo:nil repeats:YES] autorelease];
		/*
	} else { // start timer right away
		newTimer = [[newTimer initWithTimeInterval:interval target:self selector:@selector(tick:) userInfo:nil repeats:YES] autorelease];
		NSLog(@"timer initWithTimeInterval");
		*/
	}
	[[NSRunLoop currentRunLoop] addTimer:newTimer forMode:NSDefaultRunLoopMode];
	[self setTimer:newTimer];
	[timer fire];
}

- (void)stopTimer
{
	[timer invalidate];
	[self setTimer:nil];
}

- (void)tick:(id)sender
{
	// create string from current time
	NSCalendarDate *now = [NSCalendarDate calendarDate];
	NSString *s = [@" " stringByAppendingString:_am_clockSymbol];
	s = [s stringByAppendingString:[now descriptionWithCalendarFormat:@" %H:%M "]];
	// make it into an attributed string
	NSMutableAttributedString *text = [[[NSMutableAttributedString alloc] initWithString:s attributes:nil] autorelease];
	// set font for that part of the string the represents time to the system's default font
	[text addAttributes:[NSDictionary dictionaryWithObject:[NSFont systemFontOfSize:0] forKey:NSFontAttributeName] range:NSMakeRange(2, [text length]-2)];
	// create ticker sign from string
	AMTextTickerSign *newSign = [[[AMTextTickerSign alloc] initWithAttributedString:text] autorelease];
	// set parameters
	[newSign setContinuousScrolling:YES];
	// send sign to server
	// note: if the server quits unexpectedly, we will get an NSObjectInaccessibleException so we need to catch that
	@try {
		[server addSign:newSign];
	}
	@catch(NSException *exception) {
		NSLog(@"exception in tick: %@ (%@)", [exception name], [exception reason]);
		// something went wrong while talking to the server - stop timer and drop connection to server
		[self deactivate];
	}
}

- (void)applicationWillTerminate:(NSNotification *)aNotification
{
	// we need to deregister from the server ...
	[server deregisterClientWithName:@"AMTiggerClock"];
	// ... and the workspace notification center
	[[[NSWorkspace sharedWorkspace] notificationCenter] removeObserver:self];
}


@end
