//
//  TimeTickerPlugin.m
//  TimeTicker
//
//  Created by Andreas on Thu Mar 25 2004.
//  Copyright (c) 2004 Andreas Mayer. All rights reserved.
//

#import "TimeTickerPlugin.h"
#import "AMTickerService/AMTextTickerSign.h"
#import "AMTickerService/AMImageTickerSign.h"

#define AMLocalizedString(x, y) [myBundle localizedStringForKey:x value:@"" table:nil]


@interface TimeTickerPlugin (Private)
- (NSBundle *)myBundle;
- (void)setMyBundle:(NSBundle *)newMyBundle;
- (id)server;
- (void)setServer:(id)newServer;
- (NSTimer *)timer;
- (void)setTimer:(NSTimer *)newTimer;
- (NSMutableDictionary *)defaults;
- (void)setDefaults:(NSMutableDictionary *)newDefaults;
- (NSString *)timeFormat;
- (void)setTimeFormat:(NSString *)newTimeFormat;
- (void)startTimer;
- (void)presetDefaults;
- (void)updateTimeFormat;
- (AMTickerSign *)lastSign;
- (void)setLastSign:(AMTickerSign *)newLastSign;
@end


@implementation TimeTickerPlugin


// AMTiggerPluginProtcol
- (id)initWithBundle:(NSBundle *)theBundle
{
	//NSLog(@"initWithBundle: %@", theBundle);
	if (self = [super init]) {
		[self setMyBundle:theBundle];
		_am_clockSymbol = [[NSString stringWithUTF8String:"◷"] retain];
	}
	return self;
}

// AMPrefPaneProtocol

- (NSString *)identifier
{ return nil; }

- (NSView *)mainView
{
	return prefPaneView;
}

- (NSString *)label
{ return nil; }

- (NSImage *)icon
{ return nil; }

- (NSString *)category
{ return nil; }


- (NSBundle *)bundle
{
	return myBundle;
}

- (void)loadMainView
{
	if (!prefPaneView) {
		[NSBundle loadNibNamed:@"TimeTickerPane" owner:self];
	}
}

- (void)willSelect
{
	[self updateTimeFormat];
}

- (NSString *)userInfo
{
	return AMLocalizedString(@"userInfo", @"");
}

- (NSString *)version
{ return nil; }


// AMTiggerClientProtocol

- (BOOL)hasPrefsPane
{
	return YES;
}

- (void)setupWithServer:(id <AMTiggerServerProtocol>)theServer
{
	NSLog(@"setupWithServer:");
	[self setServer:theServer];
	[self setDefaults:[server preferencesForIdentifier:[myBundle bundleIdentifier]]];
	[self presetDefaults];
	[self setTimeFormat:[defaults objectForKey:@"timeFormat"]];
}

- (void)activate
{
	NSLog(@"activate");
	// start timer
	[self startTimer];
}

- (void)presetDefaults
{
	if ([defaults count] == 0) {
		[defaults setObject:@"%H:%M" forKey:@"timeFormat"];
	}
}

- (void)deactivate
{
	// stop timer
	[timer invalidate];
	[timer release];
	timer = nil;
}


- (void)dealloc
{
	[lastSign release];
	[defaults release];
	[timer invalidate];
	[timer release];
	[server release];
	[myBundle release];
	[super dealloc];
}


- (NSBundle *)myBundle
{
	return myBundle;
}

- (void)setMyBundle:(NSBundle *)newMyBundle
{
	// do not retain own bundle
	myBundle = newMyBundle;
}

- (id)server
{
	return server;
}

- (void)setServer:(id)newServer
{
	// do not retain server
	server = newServer;
}

- (NSMutableDictionary *)defaults
{
	return defaults;
}

- (void)setDefaults:(NSMutableDictionary *)newDefaults
{
	id old = nil;
	
	if (newDefaults != defaults) {
		old = defaults;
		defaults = [newDefaults retain];
		[old release];
	}
}

- (NSTimer *)timer
{
	return timer;
}

- (void)setTimer:(NSTimer *)newTimer
{
	id old = nil;
	
	if (newTimer != timer) {
		old = timer;
		timer = [newTimer retain];
		[old release];
	}
}

- (NSString *)timeFormat
{
	return timeFormat;
}

- (void)setTimeFormat:(NSString *)newTimeFormat
{
	id old = nil;
	
	if (newTimeFormat != timeFormat) {
		old = timeFormat;
		timeFormat = [newTimeFormat copy];
		[old release];
	}
}

- (AMTickerSign *)lastSign
{
	return lastSign;
}

- (void)setLastSign:(AMTickerSign *)newLastSign
{
	id old = nil;
	
	if (newLastSign != lastSign) {
		old = lastSign;
		lastSign = [newLastSign retain];
		[old release];
	}
}


- (void)startTimer
{
	float interval = 60.0;
	NSCalendarDate *startTime = [NSCalendarDate calendarDate];
	div_t divresult = div(([startTime secondOfMinute]+(int)interval), (int)interval);
	int startSecond = divresult.quot*(int)interval;
	//NSLog(@"start second for timer with interval %f: %i", interval, startSecond);
	startTime = [NSCalendarDate dateWithYear:[startTime yearOfCommonEra] month:[startTime monthOfYear] day:[startTime dayOfMonth] hour:[startTime hourOfDay] minute:[startTime minuteOfHour] second:startSecond timeZone:[startTime timeZone]];
	NSTimer *newTimer = [NSTimer alloc];
	if ([newTimer respondsToSelector:@selector(initWithFireDate:interval:target:selector:userInfo:repeats:)]) {
		newTimer = [[newTimer initWithFireDate:startTime interval:interval target:self selector:@selector(tick:) userInfo:nil repeats:YES] autorelease];
	}
	[[NSRunLoop currentRunLoop] addTimer:newTimer forMode:NSDefaultRunLoopMode];
	[self setTimer:newTimer];
	[timer fire];
}

- (void)stopTimer
{
	[timer invalidate];
	[self setTimer:nil];
}

- (void)tick:(id)sender
{
	// create string from current time
	NSCalendarDate *now = [NSCalendarDate calendarDate];
	NSString *s = [@" " stringByAppendingString:_am_clockSymbol];
	s = [s stringByAppendingString:[now descriptionWithCalendarFormat:timeFormat  locale:[[NSUserDefaults standardUserDefaults] dictionaryRepresentation]]];
	// make it into an attributed string
	NSMutableAttributedString *text = [[[NSMutableAttributedString alloc] initWithString:s attributes:nil] autorelease];
	// set font for that part of the string the represents time to the system's default font
	[text addAttributes:[NSDictionary dictionaryWithObject:[NSFont systemFontOfSize:0] forKey:NSFontAttributeName] range:NSMakeRange(2, [text length]-2)];
	// create ticker sign from string
	AMTextTickerSign *newSign = [[[AMTextTickerSign alloc] initWithAttributedString:text] autorelease];
	// set parameters
	[newSign setContinuousScrolling:YES];
	// remove last sign if not yet displayed
	[server removeSign:lastSign];
	[self setLastSign:newSign];
	// send sign to server
	[server addSign:newSign];
}


- (IBAction)changeTimeFormat:(id)sender
{
	[self setTimeFormat:[sender stringValue]];
	[self updateTimeFormat];
	[defaults setObject:timeFormat forKey:@"timeFormat"];
}

- (void)updateTimeFormat
{
	[timeFormatTextField setStringValue:timeFormat];
	[previewTextField setStringValue:[[NSCalendarDate date] descriptionWithCalendarFormat:timeFormat locale:[[NSUserDefaults standardUserDefaults] dictionaryRepresentation]]];
}


@end
